function results = Specification_Test_Bootstrap(Moment, CondVar, m, bootstrp_n)

% INPUTS:
%       Moment: The moment condition 
%       CondVar: Conditioning variables
%       m: Order of the polynomial 
%       bootstrp_n: Number of bootstrap samples


    n = length(Moment);
 
%% Create P(X)
    PX = NaN(n, m);    
    [CondVar, Index] = sort(CondVar);
    Moment = Moment(Index);
    [~,CondVar_rank] = ismember(CondVar,unique(CondVar));        
    CondVar_scaled = 2*(CondVar_rank - min(CondVar_rank))/(max(CondVar_rank) - min(CondVar_rank)) - 1;        
    for ind_k = 1:m
            PX(:, ind_k) = legendreP(ind_k-1, CondVar_scaled); 
    end
     
%% Calculate the supT stat
    betahat = (PX'*PX)\(PX'*Moment);

    u = Moment - PX*betahat; 

    unPX = u.*PX;
    unPX_demean = unPX - mean(unPX, 1);
    A_n = unPX_demean'*unPX_demean/n;       
    Q_n = 1/n*(PX'*PX);

    Sig_n =  Q_n\A_n/Q_n;
    
    That = (-1)*inf;
    That_vector = NaN(n, 1);
    for ind_n = 1:n
       PX_tmp = PX(ind_n, :);
       sigma_n = sqrt(PX_tmp * Sig_n *PX_tmp');
       T_tmp = abs(sqrt(n)*PX_tmp*betahat/sigma_n);
       That_vector(ind_n) = T_tmp;
       if That < T_tmp
          That = T_tmp; 
       end
    end

    
%% Compute the Critical Value using Bootstrap   
    That_save = NaN(bootstrp_n, 1);
    parfor ind_bootstr = 1:bootstrp_n
        
        rng(ind_bootstr*n)
        
        IDs = randsample(n,n, true); 
        PX_bstr = PX(IDs, :);
        Moment_bstr = Moment(IDs, :);
               
        betahat_bstr = (PX_bstr'*PX_bstr)\(PX_bstr'*Moment_bstr);

        u = Moment_bstr - PX_bstr*betahat_bstr; 

        unPX = u.*PX_bstr;
        unPX_demean = unPX - mean(unPX, 1);
        A_n = unPX_demean'*unPX_demean/n;       
        Q_n = 1/n*(PX_bstr'*PX_bstr);

        Sig_n =  Q_n\A_n/Q_n;       
        
        That_save(ind_bootstr) = (-1)*inf;
        for ind_n = 1:n
            PX_tmp = PX_bstr(ind_n, :);
            sigma_n = sqrt(PX_tmp * Sig_n *PX_tmp');
            T_tmp = abs(sqrt(n)*PX_tmp*(betahat_bstr-betahat)/sigma_n);
            if That_save(ind_bootstr) < T_tmp
                That_save(ind_bootstr) = T_tmp; 
            end
        end
     
    end   
    cv_save = prctile(That_save, 95);
    
    results.cv = cv_save;
    results.That = That;
    results.That_save = That_save;
    if That > cv_save
        results.decision = 0;
    else
        results.decision = 1;        
    end
    
end