clear; clc; close all;

options  =  optimset('fminunc');
options  =  optimset(options, 'Display', 'off');
addpath('./tarch')

omega_ = 0.05;
beta_ = 0.9;
gamma_ = 0.05; 
sigma_sqr0 = 1;
sig2_error = 1;

n = 2000;
sim_number = 10000;
bootstrp_n = 1000; % simulation for T stat

qs = [0.75 0.9 0.95 0.99];
for ind_q = 1:length(qs)
  
    q = qs(ind_q);
    data_name = ['DGP_N_pct' num2str(q*100)];

    m_vector = [6 10];
    Ind_Gauss_save = NaN(1, length(m_vector));
    Ind_BootStr_save = NaN(1, length(m_vector));
    m_counter_tmp = 1;
    for m = m_vector

        Ind_bstr = NaN(sim_number, 1);
        Ind_gauss = NaN(sim_number, 1);

        parfor ind_sim = 1:sim_number
            rng(ind_sim*n)

            eps = normrnd(0, sqrt(sig2_error), n+1, 1);

            Y = NaN(n+1, 1);
            Volatility_sqr = NaN(n+1, 1);
            Volatility_sqr(1) = sigma_sqr0; 
            Y(1) = Volatility_sqr(1).^(1/2)*eps(1);
            for ind_n = 2:n+1 
                Volatility_sqr(ind_n)  = omega_ + beta_*Volatility_sqr(ind_n-1) + gamma_*Y(ind_n-1)^2;
                Y(ind_n) = Volatility_sqr(ind_n).^(1/2)*eps(ind_n);      
            end

            [parameters] = tarch(Y(1:end-1),1,0,1,[],[],[],options);

            Constant_ = parameters(1);
            arch_ = parameters(2);
            garch_ = parameters(3);
            
            volhat = NaN(n, 1);
            volhat(1) = var(Y(1:end-1));
            for ind_n = 2:n
                volhat(ind_n) = Constant_ + arch_*Y(ind_n-1)^2 + volhat(ind_n-1)*garch_;
            end

            VaRhat = norminv(q)*sqrt(Constant_ + arch_*Y(1:end-1).^2 + volhat*garch_);

            Moment = ((Y(2:end) <= VaRhat) - q);
            CondVar = [Y(1:end-1) volhat.^(1/2)];     

            results = Specification_Test_2dcontrol(Moment, CondVar, m, bootstrp_n);

            Ind_bstr(ind_sim) = results.decision_bstr;
            Ind_gauss(ind_sim) = results.decision_gauss;

        end
        

        Ind_Gauss_save(:, m_counter_tmp)  = mean(Ind_gauss);
        Ind_BootStr_save(:, m_counter_tmp) = mean(Ind_bstr);

        m_counter_tmp  = m_counter_tmp+1;
        disp(['Degree:' num2str(m) ' q: ' num2str(q) ])
        save(data_name)

    end

    save(data_name)
end

