function results = Specification_Test_2dcontrol(Moment, CondVar, m, bootstrp_n)

% INPUTS:
%       Moment: The moment condition 
%       CondVar: Conditioning variables
%       m: Order of the polynomial 
%       bootstrp_n: Number of bootstrap samples


    n = length(Moment);
 
%% Create P(X)
    PX = NaN(n, m);
    [CondVar_1, Index_] = sort(CondVar(:, 1));
    Moment = Moment(Index_);
    CondVaR_2 = CondVar(Index_, 2);     
    [~,CondVar_rank_1] = ismember(CondVar_1,unique(CondVar_1));        
    CondVar_scaled_1 = 2*(CondVar_rank_1 - min(CondVar_rank_1))/(max(CondVar_rank_1) - min(CondVar_rank_1)) - 1;
   
    [~,CondVar_rank_2] = ismember(CondVaR_2,unique(CondVaR_2));        
    CondVar_scaled_2 = 2*(CondVar_rank_2 - min(CondVar_rank_2))/(max(CondVar_rank_2) - min(CondVar_rank_2)) - 1;
 
    PX(:, 1) = 1;
    degree_1 = 1;
    for ind_k = 2:3:m
            PX(:, ind_k) = legendreP(degree_1, CondVar_scaled_1); 
            degree_1 = degree_1 + 1;
    end

    degree_2 = 1;
    for ind_k = 3:3:m
            PX(:, ind_k) = legendreP(degree_2, CondVar_scaled_2); 
            degree_2 = degree_2 + 1;            
    end

    for ind_k = 4:3:m
        if ind_k == 4
             PX(:, ind_k) = PX(:, 2).*PX(:, 3);          
        elseif ind_k == 7
             PX(:, ind_k) = PX(:, 5).*PX(:, 3);            
        elseif ind_k == 10
              PX(:, ind_k) = PX(:, 2).*PX(:, 6);          
        end
    end
     
%% Calculate the supT stat
    betahat = (PX'*PX)\(PX'*Moment);

    u = Moment - PX*betahat; 

    unPX = u.*PX;
    unPX_demean = unPX - mean(unPX, 1);
    A_n = unPX_demean'*unPX_demean/n;       
    Q_n = 1/n*(PX'*PX);

    Sig_n =  Q_n\A_n/Q_n;
    
    That = (-1)*inf;
    That_vector = NaN(n, 1);
    for ind_n = 1:n
       PX_tmp = PX(ind_n, :);
       sigma_n = sqrt(PX_tmp * Sig_n *PX_tmp');
       T_tmp = abs(sqrt(n)*PX_tmp*betahat/sigma_n);
       That_vector(ind_n) = T_tmp;
       if That < T_tmp
          That = T_tmp; 
       end
    end

    
%% Compute the Critical Value using Bootstrap
    
    That_save = NaN(bootstrp_n, 1);
    parfor ind_bootstr = 1:bootstrp_n
        
        rng(ind_bootstr*n)
        
        IDs = randsample(n,n, true); 
        PX_bstr = PX(IDs, :);
        Moment_bstr = Moment(IDs, :);
               
        betahat_bstr = (PX_bstr'*PX_bstr)\(PX_bstr'*Moment_bstr);

        u = Moment_bstr - PX_bstr*betahat_bstr; 

        unPX = u.*PX_bstr;
        unPX_demean = unPX - mean(unPX, 1);
        A_n = unPX_demean'*unPX_demean/n;       
        Q_n = 1/n*(PX_bstr'*PX_bstr);

        Sig_n =  Q_n\A_n/Q_n;       
        
        That_save(ind_bootstr) = (-1)*inf;
        for ind_n = 1:n
            PX_tmp = PX_bstr(ind_n, :);
            sigma_n = sqrt(PX_tmp * Sig_n *PX_tmp');
            T_tmp = abs(sqrt(n)*PX_tmp*(betahat_bstr-betahat)/sigma_n);
            if That_save(ind_bootstr) < T_tmp
                That_save(ind_bootstr) = T_tmp; 
            end
        end
     
    end   
    cv_save = prctile(That_save, 95);
    
    results.cv_bstr = cv_save;
    results.That = That;
    results.That_save = That_save;
    if That > cv_save
        results.decision_bstr = 0;
    else
        results.decision_bstr = 1;        
    end
%% Compute Critical Value using the Gaussian Approximation Method
    betahat = (PX'*PX)\(PX'*Moment);

    u = Moment - PX*betahat; 

    unPX = u.*PX;
    unPX_demean = unPX - mean(unPX, 1);
    A_n = unPX_demean'*unPX_demean/n;       
    Q_n = 1/n*(PX'*PX);

    Sig_n =  Q_n\A_n/Q_n;
    Sig_sqrt = Sig_n^(1/2);

    T_n = zeros(bootstrp_n, 1);
    for ind_boot = 1:bootstrp_n
        xi = randn(m, 1);
        T_tmp = (-1)*inf;
        for ind_n = 1:n
            PX_tmp = PX(ind_n, :);
            sigma_n = sqrt(PX_tmp * Sig_n *PX_tmp');
            T_n_ = abs(PX_tmp * Sig_sqrt * xi/sigma_n);
            if T_n_ > T_tmp
                T_tmp = T_n_;
            end
        end

        T_n(ind_boot) = T_tmp;
    end
    cv_save = prctile(T_n, 95);
    
    results.cv_gauss = cv_save;
    if That > cv_save
        results.decision_gauss = 0;
    else
        results.decision_gauss = 1;        
    end
    
end